<?php

namespace Drupal\Tests\project_wiki\Functional;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * This class provides methods specifically for testing something.
 *
 * @group project_wiki_markdown_content
 */
class ProjectWikiListjsTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'listjs',
    'listjs_cdn_alter',
    'project_wiki',
    'project_wiki_entity_content',
    'text',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->user = $this->drupalCreateUser([]);
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);

    // Prepare the test page:
    $page = $this->getSession()->getPage();
    // Create three test entries that will later be searched for and sorted.
    $this->drupalGet('/project-wiki-entity-content/add');
    $page->fillField('edit-category-0-value', 'Category A');
    $page->fillField('edit-title-0-value', 'Title A');
    $page->fillField('edit-body-0-value', 'The content is unimportant yet required here. This content however contains 1234567890 for testing the search.');
    $page->pressButton('edit-submit');
    $this->drupalGet('/project-wiki-entity-content/add');
    $page->fillField('edit-category-0-value', 'Category B');
    $page->fillField('edit-title-0-value', 'Title B');
    $page->fillField('edit-body-0-value', 'The content is unimportant yet required here.');
    $page->pressButton('edit-submit');
    $this->drupalGet('/project-wiki-entity-content/add');
    $page->fillField('edit-category-0-value', 'Category C');
    $page->fillField('edit-title-0-value', 'Title C');
    $page->fillField('edit-body-0-value', 'The content is unimportant yet required here.');
    $page->checkField('edit-isdevelopercontent-value');
    $page->pressButton('edit-submit');
  }

  /**
   * Tests the search functionality of the ListJS library.
   */
  public function testListjsSearch() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();

    // Go to the list page:
    // Test if listjs is attached to the list correctly:
    $this->drupalGet('/admin/help/project_wiki');

    // Check they're there:
    $session->pageTextContainsOnce('Title A');
    $session->pageTextContainsOnce('Title B');
    $session->pageTextContainsOnce('Title C');

    // Check if searching for title works.
    $page->fillField('listjs-search', 'Title B');
    $session->pageTextNotContains('Title A');
    $session->pageTextContainsOnce('Title B');
    $session->pageTextNotContains('Title C');

    // Check if searching for category works.
    $page->fillField('listjs-search', 'Category C');
    $session->pageTextNotContains('Title A');
    $session->pageTextNotContains('Title B');
    $session->pageTextContainsOnce('Title C');

    // Check if searching for content works.
    $page->fillField('listjs-search', '1234567890');
    $session->pageTextContainsOnce('Title A');
    $session->pageTextNotContains('Title B');
    $session->pageTextNotContains('Title C');

    // Check if searching for developer content works.
    $page->fillField('listjs-search', 'Yes');
    $session->pageTextNotContains('Title A');
    $session->pageTextNotContains('Title B');
    $session->pageTextContainsOnce('Title C');
  }

  /**
   * Tests the sort functionality of the ListJS library.
   */
  public function testListjsSort() {
    // @todo Implement this
  }

}
